@@ -5,7 +5,7 @@
 Copyright (c) 2001-2011 Twisted Matrix Laboratories <twisted-python@twistedmatrix.com>
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-SPDX-License-Identifier: GPL-2.0
+SPDX-License-Identifier: GPL-2.0-only
 
 """
 
@@ -65,7 +65,7 @@ class TwistedSocketNotifier(QObject):
     """
 
     def __init__(self, parent:Optional[QObject], reactor:'QtReactor', watcher:unix.Client, socketType:QSocketNotifier.Type) ->None:
-        QObject.__init__(self, parent)
+        super().__init__(parent)
         self.reactor = reactor
         self.watcher:Optional[unix.Client] = watcher
         fd = watcher.fileno()
