@@ -6,7 +6,7 @@ Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 partially based on C++ code from:
  - Copyright (C) 2006 Mauricio Piacentini <mauricio@tabuleiro.com>
 
-SPDX-License-Identifier: GPL-2.0
+SPDX-License-Identifier: GPL-2.0-only
 
 """
 
@@ -15,7 +15,8 @@ import traceback
 import datetime
 import random
 import sqlite3
-from typing import List, Tuple, Union, Any, Optional, Literal
+from collections import namedtuple
+from typing import List, Tuple, Union, Any, Optional, Literal, Generator
 
 from mi18n import i18n, i18ncE
 from util import Duration
@@ -27,7 +28,7 @@ class QueryException(Exception):
     """as the name says"""
 
     def __init__(self, msg:str) ->None:
-        Exception.__init__(self, msg)
+        super().__init__(msg)
 
 
 class DBCursor(sqlite3.Cursor, ReprMixin):
@@ -35,7 +36,7 @@ class DBCursor(sqlite3.Cursor, ReprMixin):
     """logging wrapper"""
 
     def __init__(self, dbHandle:'DBHandle') ->None:
-        sqlite3.Cursor.__init__(self, dbHandle)
+        super().__init__(dbHandle)
         self.statement:str
         self.parameters:Optional[Tuple[Union[str, int], ...]] = None
 
@@ -52,9 +53,9 @@ class DBCursor(sqlite3.Cursor, ReprMixin):
                     sqlite3.Cursor.executemany(
                         self, statement, parameters)
                 elif parameters:
-                    sqlite3.Cursor.execute(self, statement, parameters)
+                    super().execute(statement, parameters)
                 else:
-                    sqlite3.Cursor.execute(self, statement)
+                    super().execute(statement)
         except sqlite3.Error as exc:
             msg = f"{self!r}: ERROR {exc}"
             if mayFail:
@@ -83,7 +84,7 @@ class DBHandle(sqlite3.Connection, ReprMixin):
         self.path = path
         self.identifier = None
         try:
-            sqlite3.Connection.__init__(self, self.path, timeout=10.0)
+            super().__init__(self.path, timeout=10.0, detect_types=sqlite3.PARSE_DECLTYPES)
         except sqlite3.Error as exc:
             if hasattr(exc, 'message'):
                 msg = exc.message
@@ -103,10 +104,10 @@ class DBHandle(sqlite3.Connection, ReprMixin):
         self.inTransaction = datetime.datetime.now()
         if Debug.sql:
             logDebug('starting transaction')
-        return sqlite3.Connection.__enter__(self)
+        return super().__enter__()
 
     def __exit__(self, *args:Any) ->Literal[False]:
-        sqlite3.Connection.__exit__(self, *args)
+        super().__exit__(*args)
         if Debug.sql:
             logDebug('finished transaction')
         self.inTransaction = None
@@ -197,7 +198,8 @@ class Query(ReprMixin):
 
     def __init__(self, statement:str,
                  args:Union[None,Tuple[Union[str, int, float], ...],List[List[Union[str,int,float]]]]=None,
-                 silent:bool=False, mayFail:bool=False, failSilent:bool=False) ->None:
+                 silent:bool=False, mayFail:bool=False, failSilent:bool=False,
+                 fields:Optional[str]=None) ->None:
         """we take one sql statement.
         Do prepared queries by passing the parameters in args.
         If args is a list of lists, execute the prepared query for every sublist.
@@ -208,11 +210,17 @@ class Query(ReprMixin):
         self.records:List[List[Any]] = []
         self.statement = statement
         self.args = args
+        self.fields = fields  # a field name might be "lastname as playername"
+        self.tuplefields:Optional[List[str]] = None
+        if fields:
+            self.tuplefields = list(self.__tuple_fieldnames())
+            self.statement = self.statement.format(fields=fields)
+
         if Internal.db:
             self.cursor = Internal.db.cursor(
                 DBCursor)
             self.cursor.execute(
-                statement,
+                self.statement,
                 args,
                 silent=silent,
                 mayFail=mayFail,
@@ -228,6 +236,54 @@ class Query(ReprMixin):
         if self.records and Debug.sql:
             logDebug(f'result set:{self.records}')
 
+    def __tupleName(self) ->str:
+        """the class name of the returned tuples.
+           For now:
+
+           - use first table name from statement.
+           - If none found: use 'Fields'
+
+           Query might get an arg tupleName which would override this
+        """
+        lower = self.statement.lower()
+        parts = lower.split(' from ')
+        if len(parts) > 1:
+            return 'Query_' + parts[1].split()[0].strip().capitalize()
+        return 'Query_Fields'
+
+    def __tuple_fieldnames(self) ->Generator[str, None, None]:
+        """translates 'fieldname as myname' into 'myname' """
+        if not isinstance(self.fields, str):
+            raise ValueError(f'must be str:{self.fields}')
+        for _ in self.fields.split(','):
+            words = [x.strip() for x in _.strip().split(' ')]
+            if len(words) == 1:
+                yield words[0]
+            elif len(words) == 3 and words[1] == 'as':
+                yield words[2]
+            else:
+                raise ValueError(f'cannot parse {_} out of {self.fields}, words={words}')
+
+    def tuples(self) -> List[Any]:
+        """named tuples for query records"""
+        tupleclass = namedtuple(self.__tupleName(), self.tuplefields)  # type: ignore
+        return [tupleclass._make(x) for x in self.records]
+
+    def tuple(self) -> Any:
+        """Valid only for queries returning exactly one record"""
+        if not self.fields:
+            raise ValueError
+        if len(self.records) != 1:
+            raise ValueError(f'{self!r} did not return exactly 1 record but {len(self.records)}')
+        tupleclass = namedtuple('Fields', self.tuplefields)  # type: ignore
+        return tupleclass._make(self.records[0])
+
+    def record(self) -> Any:
+        """Valid only for queries returning exactly one record"""
+        if len(self.records) != 1:
+            raise ValueError(f'{self!r} did not return exactly 1 record but {len(self.records)}')
+        return self.records[0]
+
     def __str__(self) ->str:
         return f"{Internal.db!r}:{self.statement} {'args=' + ','.join(str(x) for x in self.args) if self.args else ''}"
 
@@ -235,24 +291,22 @@ class Query(ReprMixin):
         """how many rows were affected?"""
         return self.cursor.rowcount if self.cursor else 0
 
+    def map(self, namedTuple) ->Optional[List[Any]]:  # FIXME: remove again
+        """A list of namedtuple"""
+        if self.records is None:
+            return None
+        return [namedTuple._make(x) for x in self.records]
+
+    def column(self, idx:int) -> List[Any]:
+        """A list with only that column"""
+        return [x[idx] for x in self.records]
 
 def initDb() ->bool:
     """open the db, create or update it if needed.
     sets Internal.db."""
     PrepareDB(DBHandle.dbPath())  # create or upgrade
-    openDb()
-    return True
-
-def closeDb() ->None:
-    """if open, close"""
-    if Internal.db:
-        Internal.db.close()
-        Internal.db = None
-
-def openDb() ->None:
-    """open the db"""
-    closeDb()
     DBHandle(DBHandle.dbPath())
+    return True
 
 
 class PrepareDB:
@@ -284,8 +338,8 @@ class PrepareDB:
             scoretime text,
             won integer,
             penalty integer default 0,
-            prevailing text,
-            wind text,
+            prevailing wind,
+            wind wind,
             points integer,
             payments integer,
             balance integer"""
@@ -360,7 +414,7 @@ class PrepareDB:
         """upgrade the structure of an existing kajongg database"""
         try:
             Internal.db = DBHandle(self.path)
-            allVersions = list(['4.13.0', '8300', '8301', '8302'])
+            allVersions = list(['4.13.0', '8300', '8301', '8302', '8303'])
             assert allVersions[-1] == str(Internal.defaultPort), f'{allVersions[-1]} != {str(Internal.defaultPort)}'
             # skip versions before current db versions:
             currentVersion = self.__currentVersion()
@@ -384,6 +438,32 @@ class PrepareDB:
         """seed now is 0 instead of Null"""
         Query('UPDATE game SET seed=0 WHERE seed IS NULL', silent=True)
 
+    def __needs_type_wind(self) ->bool:
+        """helper for updating to 8303"""
+        rows = Query("PRAGMA table_info('score')").records
+        for row in rows:
+            name = row[1]
+            typ = row[2]
+            if name == 'wind' and typ != 'wind':
+                logInfo(f'score.wind has not yet type wind: {typ}')
+                return True
+            if name == 'prevailing' and typ != 'wind':
+                logInfo(f'score.prevailing has not yet type wind: {typ}')
+                return True
+        return False
+
+    def updateToVersion8303(self) ->None:
+        """new type 'wind'"""
+        if self.__needs_type_wind():
+            with Internal.db:
+                Query('ALTER TABLE score add prevailingw wind', mayFail=True)
+                Query('ALTER TABLE score add windw wind', mayFail=True)
+                Query('UPDATE score set prevailingw = prevailing, windw=wind')
+                Query('ALTER TABLE score drop prevailing')
+                Query('ALTER TABLE score drop wind')
+                Query('ALTER TABLE score rename windw to wind')
+                Query('ALTER TABLE score rename prevailingw to prevailing')
+
     @classmethod
     def sqlForCreateTable(cls, table:str) ->str:
         """the SQL command for creating 'table'"""
