@@ -5,7 +5,7 @@ Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 partially based on C++ code from:
 Copyright (C) 2006 Mauricio Piacentini <mauricio@tabuleiro.com>
 
-SPDX-License-Identifier: GPL-2.0
+SPDX-License-Identifier: GPL-2.0-only
 
 """
 
@@ -62,13 +62,17 @@ class RuleTreeItem(TreeItem):
             item = item.parent
         return item.raw
 
+    def tooltip(self) ->str:
+        """the tooltip"""
+        return ""
+
 
 class RulesetItem(RuleTreeItem):
 
     """represents a ruleset in the tree"""
 
     def __init__(self, content:Ruleset) ->None:
-        RuleTreeItem.__init__(self, content)
+        super().__init__(content)
 
     @property
     def raw(self) ->Ruleset:
@@ -97,7 +101,7 @@ class RuleListItem(RuleTreeItem):
     """represents a list of rules in the tree"""
 
     def __init__(self, content:List[ParameterRule]) ->None:
-        RuleTreeItem.__init__(self, content)
+        super().__init__(content)
 
     @property
     def raw(self) ->'RuleList':
@@ -121,7 +125,7 @@ class RuleItem(RuleTreeItem):
     """represents a rule in the tree"""
 
     def __init__(self, content:ParameterRule) ->None:
-        RuleTreeItem.__init__(self, content)
+        super().__init__(content)
 
     @property
     def raw(self) ->Rule:
@@ -217,7 +221,7 @@ class RuleModel(TreeModel):
                     font.setItalic(True)
                     result = font
             elif role == Qt.ItemDataRole.ToolTipRole:
-                assert isinstance(item, RulesetItem)
+                assert isinstance(item, RuleTreeItem), item
                 tip = f'<b></b>{i18n(item.tooltip())}<b></b>' if item else ''
                 result = tip
         return result
@@ -274,9 +278,6 @@ class EditableRuleModel(RuleModel):
 
     """add methods needed for editing"""
 
-    def __init__(self, rulesets:List[Ruleset], title:str, parent:Optional['QObject']=None) ->None:
-        RuleModel.__init__(self, rulesets, title, parent)
-
     def __setRuleData(self, column:int, content:Union[Rule, ParameterRule],
         value:Union[str, int, bool]) ->Tuple[bool, Optional[str]]:
         """change rule data in the model"""
@@ -369,7 +370,7 @@ class RuleTreeView(QTreeView):
                  btnCopy:Optional[QPushButton]=None,
                  btnRemove:Optional[QPushButton]=None,
                  btnCompare:Optional[QPushButton]=None, parent:Optional[QWidget]=None) ->None:
-        QTreeView.__init__(self, parent)
+        super().__init__(parent)
         self.name = name
         self.setObjectName('RuleTreeView')
         self.btnCopy = btnCopy
@@ -378,7 +379,8 @@ class RuleTreeView(QTreeView):
         for button in [self.btnCopy, self.btnRemove, self.btnCompare]:
             if button:
                 button.setEnabled(False)
-        self.header().setObjectName('RuleTreeViewHeader')
+        if header := self.header():
+            header.setObjectName('RuleTreeViewHeader')
         self.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
         self.ruleModel:Optional[RuleModel] = None
         self.ruleModelTest = None
@@ -435,26 +437,29 @@ class RuleTreeView(QTreeView):
         if self.btnCompare:
             self.btnCompare.setEnabled(enableCompare)
 
-    def showEvent(self, unusedEvent:'QShowEvent') ->None:
+    def showEvent(self, unusedEvent:Optional['QShowEvent']) ->None:
         """reload the models when the view comes into sight"""
         # default: make sure the name column is wide enough
         assert self.ruleModel
         if self.ruleModel.canFetchMore():
             # we want to load all before adjusting column width
             self.ruleModel.fetchMore()
-        header = self.header()
-        header.setStretchLastSection(False)
-        header.setMinimumSectionSize(-1)
-        for col in range(1, header.count()):
-            header.setSectionResizeMode(0, QHeaderView.ResizeMode.ResizeToContents)
-        header.setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
-        for col in range(header.count()):
-            self.resizeColumnToContents(col)
+        if header := self.header():
+            header.setStretchLastSection(False)
+            header.setMinimumSectionSize(-1)
+            for col in range(1, header.count()):
+                header.setSectionResizeMode(0, QHeaderView.ResizeMode.ResizeToContents)
+            header.setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
+            for col in range(header.count()):
+                self.resizeColumnToContents(col)
 
     def selectedRow(self) ->Optional[QModelIndex]:
         """return the currently selected row index (with column 0)"""
-        rows = self.selectionModel().selectedRows()
-        return rows[0] if len(rows) == 1 else None
+        if model := self.selectionModel():
+            rows = model.selectedRows()
+            if len(rows) == 1:
+                return rows[0]
+        return None
 
     def copyRow(self) ->None:
         """copy a ruleset"""
@@ -465,7 +470,8 @@ class RuleTreeView(QTreeView):
             ruleset = item.raw.copyTemplate()
             cast(RuleModel, self.model()).appendRuleset(ruleset)
             self.rulesets.append(ruleset)
-            self.selectionChanged(self.selectionModel().selection())
+            if model := self.selectionModel():
+                self.selectionChanged(model.selection())
 
     def removeRow(self) ->None:
         """removes a ruleset or a rule"""
@@ -475,18 +481,20 @@ class RuleTreeView(QTreeView):
             assert not isinstance(item.ruleset(), PredefinedRuleset)
             assert isinstance(item, RulesetItem)
             ruleset = item.ruleset()
-            self.model().removeRow(row.row(), parent=row.parent())
+            if model := self.model():
+                model.removeRow(row.row(), parent=row.parent())
             self.rulesets.remove(ruleset)
-            self.selectionChanged(self.selectionModel().selection())
+            if sel_model := self.selectionModel():
+                self.selectionChanged(sel_model.selection())
 
     def compareRow(self) ->None:
         """shows the difference between two rulesets"""
-        rows = self.selectionModel().selectedRows()
-        ruleset = cast(RuleTreeItem, rows[0].internalPointer()).raw
-        assert isinstance(ruleset, Ruleset)
-        differ = RulesetDiffer([ruleset], self.rulesets)
-        differ.show()
-        self.differs.append(differ)
+        if row0 := self.selectedRow():
+            ruleset = cast(RuleTreeItem, row0.internalPointer()).raw
+            assert isinstance(ruleset, Ruleset)
+            differ = RulesetDiffer([ruleset], self.rulesets)
+            differ.show()
+            self.differs.append(differ)
 
 
 class RulesetSelector(QWidget):
@@ -558,7 +566,7 @@ class RulesetSelector(QWidget):
         we always want to see them in full"""
         return self.sizeHint()
 
-    def showEvent(self, unusedEvent:'QShowEvent') ->None:
+    def showEvent(self, unusedEvent:Optional['QShowEvent']) ->None:
         """reload the rulesets"""
         self.refresh()
 
@@ -567,14 +575,15 @@ class RulesetSelector(QWidget):
         self.retranslateUi()
         self.rulesetView.rulesets = Ruleset.availableRulesets()
 
-    def hideEvent(self, event:'QHideEvent') ->None:
+    def hideEvent(self, event:Optional['QHideEvent']) ->None:
         """close all differ dialogs"""
         marked:List[RulesetDiffer] = []
         differs = self.rulesetView.differs
         for differ in differs:
             differ.hide()
             marked.append(differ)
-        QWidget.hideEvent(self, event)
+        if event:
+            super().hideEvent(event)
         for _ in marked:
             del differs[_]
 
